<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.0
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */

if (!defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))) . '/includes/404.php';
}

/**
 * Frame_admin
 *
 * Удаление элемента
 */
class Frame_admin
{
	/**
	 * @var object основной объект системы
	 */
	public $diafan;

	/**
	 * @var string таблица в базе данных
	 */
	public $table;

	/**
	 * @var array настройки отображения
	 */
	public $config_module = array ();

	/**
	 * @var array список модулей, подключаемый при удалении элемента или категории
	 */
	private $include_modules = array ();

	/**
	 * Конструктор класса
	 *
     * @param Initadmin $diafan
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;
	}

	/**
	 * Удаляет элемент
	 *
	 * @return boolean
	 */
	public function del_module()
	{
		// Прошел ли пользователь проверку идентификационного хэша
		if (!$this->diafan->_user->checked)
		{
			echo "{error:'HASH'}";
			return FALSE;
		}

		//проверка прав пользователя на удаление
		if ($this->diafan->admin_roles('del'))
		{
			echo "{error:'ROLES'}";
			return FALSE;
		}

		if (!empty( $_POST["id"] ))
		{
			$ids = array ( $_POST["id"] );
		}
		else
		{
			$ids = $_POST["ids"];
		}
		foreach ($ids as $id)
		{
			//тип данных - категория
			if ($this->config_module["category"])
			{
				$this->del_category($this->table, intval($id));
			}
			//тип данных - элемент
			else
			{
				$this->del_element($this->table, intval($id));
			}
		}
		if ($_POST["action"] == "trash")
		{
			$result_q = DB::query("SELECT id FROM {trash}");
			while ($row = DB::fetch_array($result_q))
			{
				$count = DB::query_result("SELECT COUNT(*) FROM  {trash_parents} WHERE parent_id=%d", $row["id"]);
				DB::query("UPDATE {trash} SET count_children=%d WHERE id=%d", $count, $row["id"]);
			}
		}

		include_once ABSOLUTE_PATH . 'plugins/json.php';
		$result["redirect"] = URL;
		echo to_json($result);

		return TRUE;
	}

	/**
	 * Удаляет категорию
	 *
	 * @param string $table название таблицы
	 * @param integer $del_id номер удаляемой записи
	 * @param integer $trash_pid номер записи в корзине, с которой связано удаление
	 * @return boolean TRUE
	 */
	private function del_category($table, $del_id, $trash_pid = 0)
	{
		$del_row = DB::fetch_array(DB::query("SELECT * FROM {" . $table . "} WHERE id=%d", $del_id));

		$trash_id = $this->del_or_trash($table, $del_id, $trash_pid);

		$this->diafan->_cache->delete("", $this->diafan->module);

		$this->del_or_trash_where("rewrite", "cat_id='" . $del_id . "' AND module_name='" . str_replace('_category', '', $table) . "'", $trash_id);
		$this->del_or_trash_where("access", "cat_id='" . $del_id . "' AND module_name='" . str_replace('_category', '', $table) . "'", $trash_id);
		$this->include_modules($table, $del_id, $trash_id, TRUE);

		$func = 'delete';
		if (method_exists($this, $func))
		{
			$this->$func($del_id, $trash_id);
		}

		if (isset( $del_row["parent_id"] ))
		{
			$this->del_or_trash_where($table . "_parents", "element_id='" . $del_id . "'", $trash_id);

			$result = DB::query("SELECT id FROM {" . $table . "} WHERE parent_id=%d", $del_id);

			while ($row = DB::fetch_array($result))
			{
				$this->del_category($table, $row["id"], $trash_id);
			}
			if (!$trash_pid && $del_row["parent_id"])
			{
				$count = DB::query_result("SELECT COUNT(*) FROM {" . $table . "} WHERE trash='0' AND parent_id=%d", $del_row["parent_id"]);
				DB::query("UPDATE {" . $table . "} SET count_children=%d WHERE id=%d", $count, $del_row["parent_id"]);
			}
		}

		$table = str_replace("_category", '', $table);
		$result = DB::query("SELECT id FROM {" . $table . "} WHERE cat_id=%d", $del_id);

		while ($row = DB::fetch_array($result))
		{
			$this->del_element($table, $row["id"], $trash_id);
		}

		if ($this->config_module["category_rel"])
		{
			$this->del_or_trash_where($table . "_category_rel", "cat_id='" . $del_id . "'", $trash_id);
		}
		return TRUE;
	}

	/**
	 * Удаляет элементы в категории
	 *
	 * @param string $table название таблицы
	 * @param integer $del_id номер удаляемой записи
	 * @param integer $trash_pid номер записи в корзине, с которой связано удаление
	 * @return boolean TRUE
	 */
	private function del_element($table, $del_id, $trash_pid = 0)
	{
		$del_row = DB::fetch_array(DB::query("SELECT * FROM {" . $table . "} WHERE id=%d", $del_id));

		$trash_id = $this->del_or_trash($table, $del_id, $trash_pid);

		if ($table == "site")
		{
			$this->del_or_trash_where("rewrite", "site_id='" . $del_id . "' AND module_name='" . $table . "'", $trash_id);
		}
		else
		{
			$this->del_or_trash_where("rewrite", "element_id='" . $del_id . "' AND module_name='" . $table . "'", $trash_id);
		}
		$this->del_or_trash_where("access", "element_id='" . $del_id . "' AND module_name='" . $table . "'", $trash_id);

		$this->diafan->_cache->delete("", $this->diafan->module);

		$this->include_modules($table, $del_id, $trash_id);

		if (!$this->config_module["category"])
		{
			$func = 'delete';
			if (method_exists($this, $func))
			{
				$this->$func($del_id, $trash_id);
			}
		}

		if (isset( $del_row["parent_id"] ))
		{
			$this->del_or_trash_where($table . "_parents", "element_id='" . $del_id . "'", $trash_id);

			$result = DB::query("SELECT id FROM {" . $table . "} WHERE parent_id=%d", $del_id);

			while ($row = DB::fetch_array($result))
			{
				$this->del_element($table, $row["id"], $trash_id);
			}
			if ($del_row["parent_id"])
			{
				$count = DB::query_result("SELECT COUNT(*) FROM {" . $table . "} WHERE trash='0' AND parent_id=%d", $del_row["parent_id"]);
				DB::query("UPDATE {" . $table . "} SET count_children=%d WHERE trash='0' AND id=%d", $count, $del_row["parent_id"]);
			}
		}

		if (isset( $del_row["cat_id"] ) && ( $this->config_module["category_rel"] || $this->config_module["element_multiple"] ))
		{
			$this->del_or_trash_where($table . "_category_rel", "element_id='" . $del_id . "'", $trash_id);
		}
		return TRUE;
	}

	/**
	 * Помечает строку на удаление или удаляет строку из базы данных
	 *
	 * @param string $table название таблицы
	 * @param integer $del_id номер удаляемой записи
	 * @param integer $trash_id номер записи в корзине, с которой связано удаление
	 * @return integer|boolean TRUE
	 */
	protected function del_or_trash($table, $del_id, $trash_id)
	{
		if ($_POST["action"] == "trash")
		{
			DB::query("UPDATE {" . $table . "} SET trash='1' WHERE id=%d", $del_id);
			DB::query("INSERT INTO {trash} (table_name, module_name, element_id, created, parent_id, user_id) VALUES ('%s', '%s', '%d', '%d', '%d', '%d')", $table, $this->diafan->module, $del_id, time(), $trash_id, $this->diafan->_user->id);
			$parents = $this->diafan->get_parents($trash_id, "trash");
			if ($trash_id)
			{
				$parents[] = $trash_id;
			}
			$id = DB::last_id("trash");
			foreach ($parents as $parent_id)
			{
				DB::query("INSERT INTO {trash_parents} (`element_id`, `parent_id`) VALUES (%d, %d)", $id, $parent_id);
			}
			return $id;
		}
		else
		{
			$trash_id = DB::query("SELECT id FROM {trash} WHERE element_id=%d AND table_name='%s'", $del_id, $table);
			DB::query("DELETE FROM {trash} WHERE id=%d", $trash_id);
			DB::query("DELETE FROM {trash_parents} WHERE parent_id=%d OR element_id=%d", $trash_id, $trash_id);
			DB::query("DELETE FROM {" . $table . "} WHERE id=%d", $del_id);
		}
		return TRUE;
	}

	/**
	 * Помечает строки на удаление или удаляет строки из базы данных
	 *
	 * @param string $table название таблицы
	 * @param string $del_where SQL-условие для удаление записей в базе данных
	 * @param integer $trash_id номер записи в корзине, с которой связано удаление
	 * @return boolean TRUE
	 */
	public function del_or_trash_where($table, $del_where, $trash_id)
	{
		if ($_POST["action"] == "trash")
		{
			$result = DB::query("SELECT id FROM {" . $table . "} WHERE " . $del_where . " AND trash='0'");
			$parents = $this->diafan->get_parents($trash_id, "trash");
			if ($trash_id)
			{
				$parents[] = $trash_id;
			}
			while ($row = DB::fetch_array($result))
			{
				DB::query("INSERT INTO {trash} (table_name, module_name, element_id, created, parent_id, user_id) VALUES ('%s', '%s', '%d', '%d', '%d', '%d')", $table, $table, $row["id"], time(), $trash_id, $this->diafan->_user->id);
				$id = DB::last_id("trash");
				foreach ($parents as $parent_id)
				{
					DB::query("INSERT INTO {trash_parents} (`element_id`, `parent_id`) VALUES (%d, %d)", $id, $parent_id);
				}
			}
			DB::query("UPDATE {" . $table . "} SET trash='1' WHERE " . $del_where);

		}
		else
		{
			$del_ids = DB::query_result("SELECT GROUP_CONCAT(id SEPARATOR ',') FROM {" . $table . "} WHERE " . $del_where);
			$trash_id = DB::query("SELECT id FROM {trash} WHERE element_id in (%s) AND table=%h", $del_ids, $table);
			DB::query("DELETE FROM {trash} WHERE id=%d", $trash_id);
			DB::query("DELETE FROM {trash_parents} WHERE parent_id=%d OR element_id=%d", $trash_id, $trash_id);

			DB::query("DELETE FROM {" . $table . "} WHERE " . $del_where);
		}
		return TRUE;
	}

	/**
	 * Удаляет элементы из корзины
	 *
	 * @param string $table название таблицы
	 * @param integer $id номер удаляемого элемента
	 * @param integer $trash_id номер записи в корзине, с которой связано удаление
	 * @return boolean TRUE
	 */
	protected function del_from_trash($table, $id, $trash_id)
	{
		if (file_exists(ABSOLUTE_PATH . 'modules/' . $table . '/admin/' . $table . '.admin.del.php'))
		{
			include_once ABSOLUTE_PATH . 'modules/' . $table . '/admin/' . $table . '.admin.del.php';
			$func = 'del_from_trash';
			$class = ucfirst($table) . '_admin_del';
			if (method_exists($class, $func))
			{
				eval( $class . '::' . $func . '($this, $id);' );
			}
		}
		DB::query("DELETE FROM {trash} WHERE id=%d", $trash_id);
		DB::query("DELETE FROM {" . $table . "} WHERE id=%d", $id);
		DB::query("DELETE FROM {trash_parents} WHERE parent_id=%d OR element_id=%d", $trash_id, $trash_id);

		$result = DB::query("SELECT element_id, table_name, id FROM {trash} WHERE parent_id=%d", $trash_id);

		while ($row = DB::fetch_array($result))
		{
			$this->del_from_trash($row["table_name"], $row["element_id"], $row["id"]);
		}
		return TRUE;
	}

	private function include_modules($table, $del_id, $trash_id, $is_category = FALSE)
	{
		if (empty( $this->include_modules ))
		{
			if ($dirr = @opendir(ABSOLUTE_PATH . 'modules'))
			{
				while (( $module = readdir($dirr) ) !== FALSE)
				{
					if (file_exists(ABSOLUTE_PATH . 'modules/' . $module . '/admin/' . $module . '.admin.del.php'))
					{
						include_once ABSOLUTE_PATH . 'modules/' . $module . '/admin/' . $module . '.admin.del.php';
						$func = 'delete';
						$class = ucfirst($module) . '_admin_del';
						if (method_exists($class, $func))
						{
							$this->include_modules[$class] = $func;
						}
					}
				}
			}
		}
		foreach ($this->include_modules as $class => $func)
		{
			eval( $class . '::' . $func . '($this, \'' . str_replace('_category', '', $table) . "'" . ', $del_id, $trash_id, $is_category);' );
		}
	}
}
