<?php

/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.0
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */
if (!defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))) . '/includes/404.php';
}

class Initadmin extends Core
{

	/**
	 * @var integer переменная, передаваемая в URL: номер редактируемого элемента
	 */
	public $edit;

	/**
	 * @var integer переменная, передаваемая в URL: номер сохраняемого элемента
	 */
	public $save;

	/**
	 * @var integer переменная, передаваемая в URL: сохранить новый элемент (1)
	 */
	public $savenew;

	/**
	 * @var integer переменная, передаваемая в URL: номер элемента раскрытия каталога вложенности
	 */
	public $parent;

	/**
	 * @var integer переменная, передаваемая в URL: номер системного сообщения
	 */
	public $error;

	/**
	 * @var integer переменная, передаваемая в URL: добавление элемента (1)
	 */
	public $addnew;

	/**
	 * @var integer переменная, передаваемая в URL: номер категории
	 */
	public $cat;

	/**
	 * @var integer переменная, передаваемая в URL: номер страницы
	 */
	public $site;

	/**
	 * @var integer номер страницы
	 */
	public $page;

	/**
	 * @var string ЧПУ текущей страницы
	 */
	public $rewrite;

	/**
	 * @var string модуль текущей страницы
	 */
	public $module;

	/**
	 * @var string название подключаемой функции
	 */
	public $action;

	/**
	 * @var string подключаемый класс
	 */
	public $include_class;

	/**
	 * @var integer номер ошибки
	 */
	public $err;

	/**
	 * @var integer номер
	 */
	public $done;

	/**
	 * @var array переменные, передаваемые в URL
	 */
	public $rewrite_variable_names = array ( 'edit', 'savenew', 'save', 'addnew', 'site', 'cat', 'parent', 'page', 'error', 'success' );
	public $ajax;

	/**
	 * @var integer номер текущей страницы
	 */
	public $cid;

	/**
	 * @var string заголовок текущей страницы
	 */
	public $name;

	/**
	 * @var string контент текущей страницы
	 */
	public $text;

	/**
	 * @var integer номер страницы-родителя
	 */
	public $parent_id;

	/**
	 * @var User работа с пользователями
	 */
	public $_user;

	/**
	 * @var Cache кэширование
	 */
	public $_cache;

	/**
	 * @var Route ЧПУ
	 */
	public $_route;

	/**
	 * @var Template шаблоны
	 */
	public $_tpl;

	/**
	 * @var array объекты модулей
	 */
	private $_objects;

    /**
     * Конструктор класса. Определяет свойства класса
     *
     * @return \Initadmin
     */
	public function __construct()
	{
		$this->_user = new User();

		Customization::inc('includes/route.php');
		$this->_route = new Route( $this );

		Customization::inc('includes/template.php');
		$this->_tpl = new Template( $this );
	}


	/**
	 * Подключает вспомогательные модули, если они не подключены
	 *
	 * @return boolean TRUE
	 */
	public function __get($name)
	{
		if (!isset( $this->_objects[$name] ))
		{
			// подключаем обработчик запросов модуля
			if (substr($name, 0, 1) == '_')
			{
				$module = substr($name, 1);
				if (file_exists(ABSOLUTE_PATH . 'modules/' . $module . '/' . $module . '.view.php'))
				{
					include_once ABSOLUTE_PATH . 'modules/' . $module . '/' . $module . '.view.php';
				}
				if (file_exists(ABSOLUTE_PATH . 'modules/' . $module . '/' . $module . '.inc.php'))
				{
					Customization::inc('modules/' . $module . '/' . $module . '.inc.php');
					$class = ucfirst($module) . '_inc';
					$this->_objects[$name] = new $class( $this );
				}
			}
			else
			{
				return '';
			}
		}
		return $this->_objects[$name];
	}

	/**
	 * Инициализирует генерирование страницы
	 *
	 * @return boolean TRUE
	 */
	public function init()
	{
		Dev::set_profiling();

		Session::init();

		$this->include_other_file();

		Customization::inc('includes/cache.php');
		$this->_cache = new Cache;
		$this->magic_quote_off();

		define( 'BASE_PATH', "http://" . $_SERVER["HTTP_HOST"] . "/" . ( REVATIVE_PATH ? REVATIVE_PATH . '/' : '' ) );

		require_once(ABSOLUTE_PATH.'plugins/IDNA.php');
		$IDN = new idna_convert(array('idn_version' => '2008')); 
		$domain = $IDN->decode($_SERVER["HTTP_HOST"]);
		define('BASE_URL', ($domain ? $domain : $_SERVER["HTTP_HOST"]) . ( REVATIVE_PATH ? '/' . REVATIVE_PATH : '' ) );

		$this->get_language_array();
		$this->get_user();

		//Gzip::init_gzip();

		$this->reminding();
		$this->prepare_rewrite();
		$this->prepare_variables();

		$this->get_page_in_site();

		$this->get_headers();
		$this->get_module();

		Dev::get_profiling();
		//Gzip::do_gzip();
		return TRUE;
	}

	/**
	 * Подключает файлы для решения нестандартых задач (вывод каптчи...)
	 *
	 * @return boolean TRUE
	 */
	private function include_other_file()
	{
		if (!empty( $_GET["rewrite"] ))
		{
			$rewrite_array = explode("/", $_GET["rewrite"]);
			if (count($rewrite_array) > 1 && !in_array($rewrite_array[1], array ( 'model', 'view', 'admin', 'inc', 'ajax' )) && file_exists(ABSOLUTE_PATH . 'modules/' . $rewrite_array[0] . '/' . $rewrite_array[0] . '.' . $rewrite_array[1] . '.php'))
			{
				$path = ABSOLUTE_PATH . 'modules/' . $rewrite_array[0] . '/' . $rewrite_array[0] . '.' . $rewrite_array[1] . '.php';
				unset( $rewrite_array[0] );
				unset( $rewrite_array[1] );
				$_GET["rewrite"] = implode('/', $rewrite_array);
				include_once $path;
			}
		}
		return TRUE;
	}

	/**
	 * Записывает данные о языках сата
	 *
	 * @return boolean TRUE
	 */
	private function get_language_array()
	{
		unset( $_lang );
		$this->language_all();
		include_once ABSOLUTE_PATH . 'language/' . LANGUAGE_BASE . '.php';

		if ($_GET["rewrite"])
		{
			$rew = explode('/', $_GET["rewrite"], 2);
			foreach ($this->language_all as $l => $langarr)
			{
				if (in_array($rew[0], $langarr))
				{
					$_GET["rewrite"] = preg_replace('/^' . $rew[0] . '(\/)*/', '', $_GET["rewrite"]);
					define( '_LANG', $langarr[0] );
					define( '_SHORTNAME', $langarr[1] . '/' );
					define( 'TITLE', ( defined('TIT' . $langarr[0]) ? constant('TIT' . $langarr[0]) : '' ) );
					define( 'BASE_PATH_HREF', "http://" . $_SERVER["HTTP_HOST"] . "/" . ( REVATIVE_PATH ? REVATIVE_PATH . '/' : '' ) . ADMIN_FOLDER . '/' . $langarr[1] . '/' );
					break;
				}
			}
		}
		if (!defined('_LANG'))
		{
			define( '_LANG', '' );
			define( 'TITLE', TIT );
			define( '_SHORTNAME', '' );
			define( 'BASE_PATH_HREF', "http://" . $_SERVER["HTTP_HOST"] . "/" . ( REVATIVE_PATH ? REVATIVE_PATH . '/' : '' ) . ADMIN_FOLDER . '/' );
		}
		return TRUE;
	}

	/**
	 * Инициирует авторизацию или выход пользователя из системы
	 *
	 * @return boolean TRUE
	 */
	private function get_user()
	{
		if (!empty( $_POST['action'] ) && $_POST['action'] == 'auth')
		{
			$this->_user->auth($_POST);
		}
		elseif (strstr($_GET["rewrite"], "logout"))
		{
			$this->_user->logout();
		}
		return TRUE;
	}

	/**
	 * Выводит форму восстановления пароля
	 *
	 * @return boolean TRUE
	 */
	private function reminding()
	{

		if (empty( $this->_user->id ) && !empty( $_GET["rewrite"] ) && ( preg_match("/^reminding/",$_GET["rewrite"]) ))
		{
			$this->rewrite = $_GET["rewrite"];
			$this->module = "reminding";
			Customization::inc('adm/includes/theme.php');
			Customization::inc('includes/model.php');
            Customization::inc('includes/ajax.php');
			Customization::inc('adm/includes/reminding.php');
			Customization::inc('modules/reminding/reminding.ajax.php');
			$reminding = new Reminding_admin( $this );
			$reminding->form();
			$theme = new Theme_admin( $this );
			include_once ABSOLUTE_PATH . 'adm/themes/adminreminding.php';
			exit;
		}
		return TRUE;
	}

	/**
	 * Подготавливает запрос для идентифицикации страницы в таблице {site} по rewrite или по id,
	 * удаляет из строки запроса $_GET[rewrite] переданные переменные
	 *
	 * @return boolean TRUE
	 */
	private function prepare_rewrite()
	{
		if ($_GET["rewrite"])
		{
			$rewrite_array = explode("/", $_GET["rewrite"]);

			foreach ($rewrite_array as $key => $ra)
			{

				foreach ($this->rewrite_variable_names as $name)
				{
					if (preg_match('/' . $name . '([0-9]+)/', $ra, $result))
					{
						$this->$name = $result[1];
						unset( $rewrite_array[$key] );
					}
				}
			}
			$this->rewrite = implode("/", $rewrite_array);
		}
		if (!$this->rewrite)
		{
			if (!$this->admin_roles('init', 'site') || !$this->_user->id)
			{
				$this->rewrite = 'site';
			}
			else
			{
				$result = DB::query("SELECT id, rewrite FROM {adminsite} WHERE act='1' ORDER BY id ASC");
				while ($row = DB::fetch_array($result))
				{
					if (!$this->admin_roles('init', $row["rewrite"]))
					{
						$this->rewrite = $row["rewrite"];
						break;
					}
				}
			}
		}
		return TRUE;
	}

	/**
	 * Подключает файл, выполняющий запрашиваемое действие на элементом
	 *
	 * @return boolean TRUE
	 */
	private function prepare_variables()
	{
		define( 'URL', path_module($this) );

		Customization::inc("adm/includes/theme.php");

		if (!empty( $_POST["action"] ))
		{
			// подключаем обработчик запросов модуля
			if (!empty( $_POST["module"] ))
			{
				$module = preg_replace('/[^a-z0-9_]+/', '', $_POST["module"]);
				if (file_exists(ABSOLUTE_PATH . 'modules/' . $module . '/admin/' . $module . '.admin.ajax.php'))
				{
					Customization::inc("adm/includes/ajax.php");

					Customization::inc('modules/' . $module . '/admin/' . $module . '.admin.ajax.php');
					$this->include_class = ucfirst($module) . '_admin_ajax';
					$this->action = 'ajax_module';
					return TRUE;
				}
			}
			switch ($_POST["action"])
			{
				case 'trash':
				case 'delete':
					Customization::inc("adm/includes/del.php");
					$this->action = 'del_module';
					return TRUE;

				case 'restore':
					$this->action = 'restore_module';
					Customization::inc("adm/includes/del.php");
					return TRUE;

				case 'move':
					$this->action = 'move_module';
					Customization::inc("adm/includes/move.php");
					return TRUE;

				case 'move_parent':
					$this->action = 'move_parent_module';
					Customization::inc("adm/includes/move.php");
					return TRUE;

				case 'move_page':
					$this->action = 'move_page';
					Customization::inc("adm/includes/move.php");
					return TRUE;

				case 'element':
					$this->action="group_cat_id";
					Customization::inc("/adm/includes/ajax.php");
					return TRUE;

				case 'unblock':
				case 'block':
					Customization::inc("adm/includes/act.php");
					$this->action = 'act_module';
					return TRUE;

				case 'sort':
					Customization::inc("adm/includes/ajax.php");
					$this->action = 'sort_module';
					return TRUE;

				case 'fast_save':
					Customization::inc("adm/includes/ajax.php");
					$this->action = 'fast_save';
					return TRUE;

				case 'parent':
					Customization::inc("adm/includes/ajax.php");
					$this->action = 'parent_id_module';
					return TRUE;

				case 'change_theme':
					Customization::inc("adm/includes/ajax.php");
					$this->action = 'change_theme';
					return TRUE;
			}


		}

		if (!empty( $_POST['ajax'] ) && isset($_SERVER['HTTP_X_REQUESTED_WITH']) && !empty($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest')
		{
			$this->ajax = $_POST['ajax'];
		}

		if ($this->save)
		{
			Customization::inc("adm/includes/save.php");
			$this->action = 'save_module';
			return TRUE;
		}

		if ($this->savenew && !empty( $_POST["id"] ))
		{
			$this->save = intval($_POST["id"]);
			Customization::inc("adm/includes/save.php");
			$this->action = 'save_module';
			return TRUE;
		}

		if ($this->savenew)
		{
			Customization::inc("adm/includes/save.php");
			$this->action = 'new_module';
			return TRUE;
		}

		if ($this->edit || $this->addnew || strstr($this->rewrite, "config"))
		{
			if (!$this->edit)
			{
				$this->edit = 1;
			}
			Customization::inc("adm/includes/edit.php");
			$this->action = 'show_theme';
		}
		else
		{
			Customization::inc("adm/includes/show.php");
			$this->action = 'show_theme';
		}
		return TRUE;
	}

	/**
	 * Идентифицирует страницу, задает параметры страницы
	 *
	 * @return boolean TRUE
	 */
	private function get_page_in_site()
	{
		if ($this->rewrite)
		{
			$row = DB::fetch_array(DB::query("SELECT id, name, text, parent_id FROM {adminsite} WHERE rewrite = '%h'", $this->rewrite));
		}

		if (empty( $row ))
		{
			include ABSOLUTE_PATH . 'includes/404.php';
		}

		$this->cid = $row["id"];
		$this->name = $row['name'];
		$this->text = $row['text'];
		$this->parent_id = $row['parent_id'];

		return TRUE;
	}

	/**
	 * Отдает браузеру заголовки страницы
	 *
	 * @return boolean TRUE
	 */
	private function get_headers()
	{
		header("Expires: " . date("r"));
		header("Cache-Control: no-store, no-cache, must-revalidate");
		header("Pragma: no-cache");
		header("Cache-Control: post-check=0, pre-check=0", FALSE);
		header('Content-Type: text/html; charset=utf-8');

		return TRUE;
	}

	/**
	 * Подключает модуль
	 *
	 * @return boolean TRUE
	 */
	private function get_module()
	{
		if ($this->admin_roles('init') && $this->_user->id)
		{
			include ABSOLUTE_PATH . 'includes/404.php';
		}
		if (strstr($this->rewrite, '/'))
		{
			$rew = explode('/', $this->rewrite);
			foreach ($rew as $k => $v)
			{
				$v = preg_replace('/[^A-Za-z0-9-]+/', '', $v);
				if ($k == 0)
				{
					$rewrite = $v;
					$module = $v . '.admin';
				}
				else
				{
					$module .= '.' . $v;
				}
			}
		}
		else
		{
			$rewrite = preg_replace('/[^A-Za-z0-9-]+/', '', $this->rewrite);
			$module = $rewrite . '.admin';
		}

		if (file_exists(ABSOLUTE_PATH . 'modules/' . $rewrite . '/admin/' . $module . '.php') && $this->_user->id)
		{
			if (!empty( $_GET["parent"] ))
			{
				$this->parent = preg_replace('/[^0-9]+/', '', $_GET["parent"]);
			}

			Customization::inc('modules/' . $rewrite . '/admin/' . $module . '.php');
		}
		$this->module = $rewrite;
		$name_class_module = str_replace('.', '_', ucfirst($module));
		$name_func_module = 'inc_file_' . $rewrite;

		if (function_exists($name_func_module))
		{
			$name_class_module = $name_func_module($this);
		}

		if (in_array($name_class_module, get_declared_classes()))
		{
			$frame = new $name_class_module( $this );
			$this->get_config_module($frame);
		}
		else
		{
			$frame = new Frame_admin( $this );
		}

		if ($this->parent && !empty( $frame->config_module["parent"] ) && !DB::query_result("SELECT id FROM {" . $frame->table . "} WHERE id=%d", $this->parent))
		{
			$this->redirect(path_module($this, 0, 1));
			exit;
		}

		if ($this->parent && !!empty( $frame->config_module["parent"] ) && !DB::query_result("SELECT id FROM {" . $frame->table . "_category} WHERE id=%d", $this->parent))
		{
			$this->redirect(path_module($this, 0, 1));
			exit;
		}

		if ($this->site && !empty( $frame->config_module["element_site"] ) && !DB::query_result("SELECT id FROM {site} WHERE id=%d", $this->site))
		{
			$this->redirect(BASE_PATH_HREF . $this->rewrite . '/');
			exit;
		}

		if (!empty( $this->ajax ) && method_exists($frame, 'ajax_' . $this->ajax))
		{
			$f = 'ajax_' . $this->ajax;
			$frame->$f();
			exit;
		}

		if ($this->action)
		{
			$action = $this->action;
			$frame->$action();
		}

		return TRUE;
	}

	public function admin_roles($action, $module_name = '', $roles = '')
	{
		if (!$roles)
		{
			$roles = $this->_user->roles;
			if (empty( $roles ))
			{
				return TRUE;
			}
		}

		if (!empty( $roles['all'] ))
		{
			return FALSE;
		}

		if (!$module_name)
		{
			$module_name = $this->rewrite;
		}

		$res = TRUE;

		if (!empty( $roles[$module_name] ))
		{
			if ($roles[$module_name][0] == 'all' || in_array($action, $roles[$module_name]))
			{
				$res = FALSE;
			}
		}

		return $res;
	}

	/**
	 * Задает настройку модуля по умолчанию
	 *
	 * @return boolean TRUE
	 */
	public function get_config_module(&$th)
	{
		$config_module = array ( 'act', 'access', 'del', 'nopage', 'element', 'element_multiple', 'element_site', 'category', 'category_flat', 'category_no_empty', 'category_rel', 'date', 'datetime', 'parent', 'menu', 'view', 'order', 'trash', 'no_out', 'tab_card', 'search_name', 'config', 'link_to_element', 'image', 'prior' );

		foreach ($config_module as $param)
		{
			if (empty( $th->config_module[$param] ))
			{
				$th->config_module[$param] = '';
			}
		}

		return TRUE;
	}

	/**
	 * Борьба с магическими кавычками
	 *
	 * @return boolean TRUE
	 */
	private function magic_quote_off()
	{
		if (get_magic_quotes_gpc())
		{
			$_GET = $this->stripslashes_array($_GET);
			$_POST = $this->stripslashes_array($_POST);
		}
		return TRUE;
	}

	/**
	 * Вырезает магические кавычки из массива
	 *
	 * @return array
	 */
	private function stripslashes_array($array)
	{
		if (is_array($array))
		{
			foreach ($array as $key => $value)
			{
				$array[$key] = $this->stripslashes_array($value);
			}
			return $array;
		}
		else
		{
			return stripslashes($array);
		}
	}
}