<?php

/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.0
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */
if (!defined('DIAFAN'))
{
    include dirname(dirname(__FILE__)).'/includes/404.php';
}

/**
 * Ajax
 * 
 * Каркас для обработки Ajax-запросов модуля
 */
abstract class Ajax extends Model
{
    /**
     * @var string метка формы, обычно название модуля + идентификатор формы
     */
    protected $tag;
 
	/**
     * @var string модуль
     */
    protected $module;

    /**
     * @var string значения параметров из формы, сформированные для письма администратору
     */
    protected $message_param;

    /**
     * @var integer номер страницы сайта, к которой прикреплен моудуль
     */
    protected $site_id;

    /**
     * Проверяет на заполнение обязательных полей
     * 
	 * @param array $config настройки функции: module модуль, params поля формы, prefix префикс
     * @return boolean
     */
    public function empty_required_field($config)
    {
		if(empty($config["module"]) || empty($config["params"]))
		{
			return false;
		}
        $prefix =  ! empty($config["prefix"]) ? $config["prefix"] : '';
        $module =  $config["module"];
        $params =  $config["params"];

        $message = array();
        foreach ($params as $row)
        {
            if ($row["required"] && empty($_POST[$prefix.'p'.$row["id"]]))
            {
                $this->result["errors"][$prefix."p".$row["id"]] = $this->diafan->configmodules('error_empty_fields', $module, $this->site_id, _LANG).' '.$row["name"];
            }
            
            if (empty($_POST[$prefix."p".$row["id"]]) && $row["type"] != "checkbox")
            {
                continue;
            }

            if ($row["type"] == "email")
            {
                $this->valid_email($_POST[$prefix."p".$row["id"]], $prefix."p".$row["id"]);
            }

            if ($row["type"] == "text" || $row["type"] == "textarea" || $row["type"] == "email")
            {
                $_POST[$prefix."p".$row["id"]] = $this->diafan->get_param($_POST, $prefix."p".$row["id"], '', 1);
                $message[] = $row["name"].': '.$_POST[$prefix."p".$row["id"]];
            }
            elseif ($row["type"] == "numtext")
            {
                $_POST[$prefix."p".$row["id"]] = (int)$this->diafan->get_param($_POST, $prefix."p".$row["id"], '', 2);
                $message[] = $row["name"].': '.$_POST[$prefix."p".$row["id"]];
            }
            elseif ($row["type"] == "date")
            {
                $_POST[$prefix."p".$row["id"]] = $this->diafan->formate_in_date($_POST[$prefix."p".$row["id"]]);
                $message[] = $row["name"].': '.$_POST[$prefix."p".$row["id"]];
            }
            elseif ($row["type"] == "datetime")
            {
                $_POST[$prefix."p".$row["id"]] = $this->diafan->formate_in_datetime($_POST[$prefix."p".$row["id"]]);
                $message[] = $row["name"].': '.$_POST[$prefix."p".$row["id"]];
            }
            elseif ($row["type"] == "checkbox")
            {
                $value = !empty($_POST[$prefix."p".$row["id"]]) ? 1 : 0;
                if (empty($row["select_values"][$value]) && $value == 1)
                {
                    $message[] = $row["name"];
                }
                elseif(! empty($row["select_values"][$value]))
                {
                    $message[] = $row["name"].': '.$row["select_values"][$value];
                }
            }
            elseif ($row["type"] == "select" && !empty($row["select_array"]))
            {
                foreach ($row["select_array"] as $select)
                {
                    if ($select["id"] == $_POST[$prefix."p".$row["id"]])
                    {
                        $message[] = $row["name"].': '.$select["name"];
                    }
                }
            }
            elseif ($row["type"] == "multiple" && !empty($row["select_array"]) && !empty($_POST[$prefix."p".$row["id"]]) && is_array($_POST[$prefix."p".$row["id"]]))
            {
                $vals = array();
                foreach ($row["select_array"] as $select)
                {
                    if (in_array($select["id"], $_POST[$prefix."p".$row["id"]]))
                    {
                        $vals[] = $select["name"];
                    }
                }
                $message[] = $row["name"].': '.implode(", ", $vals);
            }
        }
        
        $this->message_param = implode('<br>', $message);
        return false;
    }

    /**
     * Добавляет значение полей формы в базу данных
     * 
	 * @param array $config настройки функции: id номер элемента, table таблица, params поля формы
     * @return boolean true
     */
    public function insert_values($config)
    {
		if(empty($config["id"]) || empty($config["table"]) || empty($config["params"]))
		{
			return false;
		}
        $id =  $config["id"];
        $table =  $config["table"];
        $params =  $config["params"];
        foreach ($params as $row)
        {
            if (empty($_POST["p".$row["id"]]))
				continue;

            if ($row["type"] == "multiple")
            {
                foreach ($_POST["p".$row["id"]] as $val)
                {
                    DB::query("INSERT INTO {".$table."_param_element} (value, param_id, element_id) VALUES ('%h', %d, %d)", $val, $row["id"], $id);
                }
            }
            else
            {
                DB::query("INSERT INTO {".$table."_param_element} (value, param_id, element_id) VALUES ('%h', %d, %d)", $_POST["p".$row["id"]], $row["id"], $id);
            }
        }
        return true;
    }

    /**
     * Проверка e-mail на валидность
     * 
     * @return boolean
     */
    public function valid_email($email, $filed)
    {
        if ($email)
        {
            Customization::inc('includes/validate.php');
            $mes = Validate::mail($email);
            if ($mes)
            {
            $this->result["errors"][$filed] = $mes;
            return true;
            }
        }
        
        return false;
    }

    /**
     * Проверяет правильность каптчи
     * 
     * @return boolean
     */
    public function check_captcha()
    {
		if (! $this->module)
		{
			$this->module = $this->diafan->module;
		}
		if (! $this->tag)
		{
			$this->tag = $this->module;
		}

		$this->result["code"] = '';

		if ($this->diafan->configmodules('captcha', $this->module, $this->site_id))
		{
			$this->result["code"] = $this->diafan->_captcha->get($this->tag, '', (bool) $_POST['update']);

			if (empty($_POST['update']))
			{
				$mes = $this->diafan->_captcha->is_right($this->tag);
				if ($mes)
				{
					$this->result["errors"]["code"] = $mes;
					return true;
				}
			}
			else
			{
				$this->result["update_code"] = true;
			}
		}
		return false;
    }

    /**
     * Отправляет ответ
     * 
     * @return boolean
     */
    public function send_errors()
    {
		if(! $this->tag)
		{
			$this->tag = $this->diafan->module;
		}
		$params = array("errors", "success", "update_code", "redirect", "form", "form_hide", "add", "data");

		$s = false;
		foreach ($params as $v)
		{
			if (!empty($this->result[$v]))
			{
				$s = true;
				break;
			}
		}

		if ($s)
		{
			if (!empty($_POST['ajax']))
			{
				echo $this->to_json($this->result);
			}
			else
			{
				$query = '';
				if (!empty($this->result["errors"]))
				{
					foreach ($this->result["errors"] as $field => $mes)
					{
					if (empty($_POST["update"]))
					{
						$query .= '&mess'.($field ? '-'.$field : '').'='.$mes;
					}
					}
				}
				$this->diafan->redirect($this->diafan->_route->current_link().($query ? '?error='.$this->tag.$query : ''));
			}
			return true;
		}
		return false;
    }

	public function to_json($result)
	{
		include_once ABSOLUTE_PATH.'plugins/json.php';
		return to_json($result);
	}
}