<?php

/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.0
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */
if (!defined('DIAFAN'))
{
    include dirname(dirname(dirname(__FILE__))) . '/includes/404.php';
}

/**
 * Registration_ajax
 *
 * Обработка запроса при регистрации или изменении данных о пользователе
 */
class Registration_ajax extends Ajax
{
    /**
     * Обрабатывает полученные данные из формы
     * 
     * @return boolean
     */
    public function ajax_request()
    {
        if (empty($_POST['module']) || $_POST['module'] != 'registration' || empty($_POST["action"]) || $this->diafan->module != 'registration')
        {
            return false;
        }

		$this->module = 'users';
		$this->tag = 'registration';

		switch($_POST["action"])
		{
			case 'addnew':
				return $this->addnew();
			case 'edit':
				return $this->edit();
			case 'delete_avatar':
				return $this->delete_avatar();
		}
		return false;
	}

	/*
	 * Добавляет нового пользователя
	 * 
	 * @return boolean
	 */
	private function addnew()
	{
        if ($this->diafan->_user->id)
        {
			return false;
		}
		$this->check_captcha();
		$this->validate();

		$params = $this->get_params(array("module" => "users", "where" => "show_in_form_no_auth='1'"));

		$this->empty_required_field(array("module" => "users", "params" => $params));

		if ($this->send_errors())
			return true;

		DB::query("INSERT INTO {users} (name, password, mail, created, language, fio, act)"
				. " VALUES ('%h', '%h', '%h', '%d', '%d', '%h', '%d')",
				$_POST["name"], code_password($_POST["password"]), $_POST["mail"], time(),
				_LANG, $_POST["fio"], $this->diafan->configmodules("act", "users") ? 0 : 1
		);
		$save_id = DB::last_id("users");

		$this->send_mails($save_id);

		$this->insert_values(array("id" => $save_id, "table" => "users", "params" => $params));

		$this->upload_avatar();

		if (!$this->diafan->configmodules("act", "users"))
		{
			$this->diafan->_user->id = $save_id;
			if ($_POST["url"])
			{
				$this->result["redirect"] = $_POST["url"];
				return $this->send_errors();
			}
		}

		$this->result["errors"][0] = $this->diafan->configmodules('mes', "users");
		$this->result["success"] = true;

		return $this->send_errors();
	}

	/*
	 * Редактирует данные пользователя
	 * 
	 * @return boolean
	 */
	private function edit()
	{
        if (! $this->diafan->_user->id)
        {
			return false;
		}
		if (!$this->diafan->_user->checked)
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}
		$this->result["hash"] = $this->diafan->_user->get_hash();

		if ($this->validate_edit())
			return true;

		$params = $this->get_params(array("module" => "users", "where" => "show_in_form_auth='1'"));
		$this->empty_required_field(array("module" => "users", "params" => $params));

		if($this->diafan->_route->id_module('cart'))
		{
			$order_params = $this->get_params(array("module" => "shop", "table" => "shop_order", "where" => "show_in_form_register='1'"));
			$this->empty_required_field(array("module" => "shop", "params" => $order_params, "prefix" => "dop_"));
		}

		if ($this->send_errors())
			return true;

		if (!empty($_POST["language"]) && !DB::query_result("SELECT id FROM {language} WHERE id=%d", $_POST["language"]))
		{
			$this->result["errors"][0] = 'ERROR';
			return $this->send_errors();
		}

		DB::query("UPDATE {users} SET name='%h', mail='%h', fio='%h' WHERE id=%d", $_POST["name"], $_POST["mail"],
				// $_POST["language"],
					$_POST["fio"], $this->diafan->_user->id
		);

		if(DB::query_result("SELECT id FROM {modules} WHERE module_name='subscribtion'") && $this->diafan->configmodules("cat", "subscribtion"))
		{
			$array = array();
			$result = DB::query("SELECT id FROM {subscribtion_category} WHERE trash='0'");
			while ($row = DB::fetch_array($result))
			{
				if (empty($_POST["subscribtion"]) || !in_array($row["id"], $_POST["subscribtion"]))
				{
					$array[] = $row["id"];
				}
			}
			$subscribtion = '0 ' . implode("  ", $array) . ' 0';

			DB::query("UPDATE {users} SET subscribtion='%s' WHERE id=%d", $subscribtion, $this->diafan->_user->id);
		}

		if ($_POST["password"])
		{
			DB::query("UPDATE {users} SET password='%h' WHERE id=%d", code_password($_POST["password"]), $this->diafan->_user->id);
		}

		$this->update_values($params);
		if(! empty($order_params))
		{
			$this->update_order_values($order_params, 'dop_');
		}

		$this->upload_avatar();

		$this->result["errors"][0] = _LANG_CHANGES_SAVED;

		return $this->send_errors();
	}

	/*
	 * Удаляет аватар пользователя
	 * 
	 * @return boolean
	 */
	private function delete_avatar()
	{
        if ($this->diafan->configmodules("avatar", "users") && $this->diafan->_user->id)
        {
            unlink(ABSOLUTE_PATH . USERFILES.'/avatar/' . $this->diafan->_user->name . '.png');
            return true;
        }
        return false;
    }

    /**
     * Проверяет валидность введенных при регистрации данных
     * 
     * @return boolean true
     */
    private function validate()
    {
        Customization::inc('includes/validate.php');
        $mes = Validate::login($_POST["name"]);
        if ($mes)
        {
            $this->result["errors"]["name"] = $mes;
        }
        $mes = Validate::mail($_POST["mail"]);
        if ($mes)
        {
            $this->result["errors"]["mail"] = $mes;
        }
        else
        {
            $mes = Validate::mail_user($_POST["mail"]);
            if ($mes)
            {
                $this->result["errors"]["mail"] = $mes;
            }
        }
        $mes = Validate::password($_POST["password"]);
        if ($mes)
        {
            $this->result["errors"]["password"] = $mes;
        }
        elseif ($_POST["password"] != $_POST["password2"])
        {
            $this->result["errors"]["password"] = _LANG_PASSWORDS_ARENT_SIMULAR;
        }

        if (!$_POST["fio"])
        {
            $this->result["errors"]["fio"] = _LANG_ENTER_NAME_SURNAME;
        }

        return true;
    }

    /**
     * Проверяет валидность введенных при редактировании данных
     * 
     * @return boolean
     */
    private function validate_edit()
    {
        Customization::inc('includes/validate.php');
        if ($_POST["name"] != $this->diafan->_user->name)
        {
            $mes = Validate::login($_POST["name"]);
            if ($mes)
            {
                $this->result["errors"]["name"] = $mes;
            }
        }
        $mes = Validate::mail($_POST["mail"]);
        if ($mes)
        {
            $this->result["errors"]["mail"] = $mes;
        }
        if ($_POST["mail"] != $this->diafan->_user->mail)
        {
            $mes = Validate::mail_user($_POST["mail"]);
            if ($mes)
            {
                $this->result["errors"]["mail"] = $mes;
            }
        }
        if ($_POST["password"])
        {
            $mes = Validate::password($_POST["password"]);
            if ($mes)
            {
                $this->result["errors"]["password"] = $mes;
            }
            elseif ($_POST["password"] != $_POST["password2"])
            {
                $this->result["errors"]["password"] = _LANG_PASSWORDS_ARENT_SIMULAR;
            }
        }

        if (empty($_POST["fio"]))
        {
            $this->result["errors"]["fio"] = _LANG_ENTER_NAME_SURNAME;
        }

        return $this->send_errors();
    }

    /**
     * Отправляет письмо новому пользователю и администратору сайта
     * 
     * @return boolean true
     */
    private function send_mails($save)
    {
        include_once ABSOLUTE_PATH . 'includes/mail.php';
        if ($this->diafan->configmodules("sendmailadmin", "users"))
        {
            $subject = str_replace(
                    array('%title', '%url'), array(TIT, BASE_URL), $this->diafan->configmodules('subject_admin', "users")
            );
            $message = str_replace(
                    array('%login', '%title', '%url', '%fio', '%email'), array(
						$this->diafan->get_param($_POST, "name", '', 1),
						TIT,
						BASE_URL,
						$this->diafan->get_param($_POST, "fio", '', 1),
						$this->diafan->get_param($_POST, "mail", '', 1)
                    ), $this->diafan->configmodules('message_admin', "users")
            );

            if ($message && $subject)
            {
                send_mail(
                        $this->diafan->configmodules("emailconfadmin", "users") ? $this->diafan->configmodules("email_admin", "users") : EMAIL_CONFIG, $subject, $message, $this->diafan->configmodules("emailconf", "users") ? $this->diafan->configmodules("email", "users") : ''
                );
            }
        }

        //send mail user
        $subject = str_replace(
                array('%title', '%url'), array(TITLE, BASE_URL), $this->diafan->configmodules('subject', "users")
        );

        $actlink = '';
        if ($this->diafan->configmodules("act", "users") == 1)
        {
            $actcode = md5(rand(111, 99999));
            DB::query("INSERT INTO {users_actlink} (link, user_id, created) VALUES ('%s', '%d', '%d')", $actcode, $save, time() + 86400);
            $link    = BASE_PATH_HREF . $this->diafan->_route->link($this->diafan->cid) . '?act=' . $actcode;
            $actlink = '<a href="' . $link . '">' . $link . '</a>';
        }

        $message = str_replace(
                array('%login', '%title', '%url', '%fio', '%password', '%actlink'), array(
					$this->diafan->get_param($_POST, "name", '', 1),
					TITLE,
					BASE_URL,
					$this->diafan->get_param($_POST, "fio", '', 1),
					$this->diafan->get_param($_POST, "password", '', 1),
					$actlink
                ), $this->diafan->configmodules('message', "users")
        );

        if ($message && $subject)
        {
            send_mail(
                    $_POST["mail"], $subject, $message, $this->diafan->configmodules("emailconf", "users") ? $this->diafan->configmodules("email", "users") : ''
            );
        }
        return true;
    }

    /**
     * Добавляет значение полей формы в базу данных
     * 
     * @return boolean true
     */
    private function update_values($params)
    {
        foreach ($params as $row)
        {
            if (empty($_POST["p" . $row["id"]]))
                continue;

            if ($row["type"] == "multiple")
            {
                DB::query("DELETE FROM {users_param_element} WHERE param_id=%d AND element_id=%d", $row["id"], $this->diafan->_user->id);
                foreach ($_POST["p" . $row["id"]] as $val)
                {
                    DB::query("INSERT INTO {users_param_element} (value, param_id, element_id) VALUES ('%h', '%d', '%d')", $val, $row["id"], $this->diafan->_user->id);
                }
            }
            elseif ($id = DB::query_result("SELECT id FROM {users_param_element} WHERE param_id=%d AND element_id=%d", $row["id"], $this->diafan->_user->id))
            {
                DB::query("UPDATE {users_param_element} SET value='%h' WHERE id=%d", $_POST["p" . $row["id"]], $id);
            }
            else
            {
                DB::query("INSERT INTO {users_param_element} (value, param_id, element_id) VALUES ('%h', '%d', '%d')", $_POST["p" . $row["id"]], $row["id"], $this->diafan->_user->id);
            }
        }
        return true;
    }

    /**
     * Добавляет значение полей формы в базу данных
     * 
     * @return boolean true
     */
    private function update_order_values($order_params, $prefix)
    {
        foreach ($order_params as $row)
        {
            if (empty($_POST[$prefix . "p" . $row["id"]]))
                continue;

            if ($row["type"] == "multiple")
            {
                DB::query("DELETE FROM {shop_order_param_user} WHERE param_id=%d AND user_id=%d", $row["id"], $this->diafan->_user->id);
                foreach ($_POST["p" . $row["id"]] as $val)
                {
                    DB::query("INSERT INTO {shop_order_param_user} (value, param_id, user_id) VALUES ('%h', '%d', '%d')", $val, $row["id"], $this->diafan->_user->id);
                }
            }
            elseif ($id = DB::query_result("SELECT id FROM {shop_order_param_user} WHERE param_id=%d AND user_id=%d", $row["id"], $this->diafan->_user->id))
            {
                DB::query("UPDATE {shop_order_param_user} SET value='%h' WHERE id=%d", $_POST[$prefix . "p" . $row["id"]], $id);
            }
            else
            {
                DB::query("INSERT INTO {shop_order_param_user} (value, param_id, user_id) VALUES ('%h', '%d', '%d')", $_POST[$prefix . "p" . $row["id"]], $row["id"], $this->diafan->_user->id);
            }
        }
        return true;
    }

    /**
     * Загружает аватар
     * 
     * @return boolean
     */
    private function upload_avatar()
    {
        if (!$this->diafan->configmodules("avatar", "users"))
        {
            return false;
        }
        if (isset($_FILES["avatar"]) && is_array($_FILES["avatar"]) && $_FILES["avatar"]['name'] != '')
        {
            list($width, $height) = getimagesize($_FILES["avatar"]['tmp_name']);
            if (!$width || !$height)
            {
                $this->result["errors"]["avatar"] = _LANG_AVATAR_INCORRECT_FILE;
                return $this->send_errors();
            }
            if ($width < $this->diafan->configmodules("avatar_width", "users") || $height < $this->diafan->configmodules("avatar_height", "users"))
            {
                $this->result["errors"]["avatar"] = sprintf(_LANG_AVATAR_INCORRECT_SIZE, $this->diafan->configmodules("avatar_width", "users"), $this->diafan->configmodules("avatar_height", "users"));
                return $this->send_errors();
            }
            Customization::inc('includes/image.php');
            if (!Image::resize($_FILES["avatar"]['tmp_name'], $this->diafan->configmodules("avatar_width", "users"), $this->diafan->configmodules("avatar_height", "users"), $this->diafan->configmodules("avatar_quality", "users"), true, true))
            {
                $this->result["errors"]["avatar"] = _LANG_AVATAR_FILE_NO_UPLOAD;
                return $this->send_json();
            }
            $dst_img  = imageCreateTrueColor($this->diafan->configmodules("avatar_width", "users"), $this->diafan->configmodules("avatar_height", "users"));
            $original = @imageCreateFromString(file_get_contents($_FILES["avatar"]['tmp_name']));
            imageCopy($dst_img, $original, 0, 0, 0, 0, $this->diafan->configmodules("avatar_width", "users"), $this->diafan->configmodules("avatar_height", "users"));
            imagePNG($dst_img, ABSOLUTE_PATH . USERFILES . '/avatar/' . $_POST["name"] . '.png');
            if ($this->diafan->_user->id)
            {
                $result["name"]          = $this->diafan->_user->name;
                $result["fio"]           = $this->diafan->_user->fio;
                $result["avatar_width"]  = $this->diafan->configmodules("avatar_width", "users");
                $result["avatar_height"] = $this->diafan->configmodules("avatar_height", "users");

                $this->result["data"] = $this->diafan->_tpl->get('avatar', 'registration', $result);
                $this->result["target"] = '.registration_avatar';
            }
        }
        return true;
    }

}