<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.0
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Search_admin_act
 * 
 * Добавление элемента в поисковый индекс. Удаление элемента из поискового индекса.
 */
class Search_admin_act
{
	protected static $config;
	
    protected static $params;
    
     
    /**
    * Установка параметров для индексирования. 
    * 
    * @return void
    */
    protected static function set_params($th, $params)
    {
        if(empty($params))
        {
            self::$params['module'] = $th->diafan->module;
            self::$params['config_module']  = $th->config_module;
            self::$params['site'] =  $th->diafan->site;
        }
        else
        {
            self::$params = $params;
        }
    }
    
    
    /**
    * Создание/удаление поискового индекса при активации/деактивации/удалении элементов.
    * 
    * @return boolean true
    */
    public static function act($th, $table, $element_ids, $act, $params = array())
	{
		if($params)
		{
			self::$params = $params;
		}
		if(! file_exists(ABSOLUTE_PATH.'modules/'.self::$params['module'].'/'.self::$params['module'].'.search.php'))
		{
			return false;
		}
        self::set_params($th, $params);

        Customization::inc('modules/search/admin/search.admin.index.php');

        if($act == 0)
        {
            self::delete_search_index($table, $element_ids);
        }
        else
        {
            self::create_search_index($th, $table, $element_ids);
        }
	  
	   return true;
	}
	
    /**
    * Удаление поискового индекса для заданных элементов модуля.
    * 
    * @param string $module_name название модуля
    * @param array $element_ids массив элементов
    * 
    * @return boolean true
    */
	protected static function delete_search_index($module_name, $element_ids)
	{
	   $lang = _LANG ? _LANG : 1;
	   foreach ($element_ids as $element_id)
	   {
		  Search_index::delete($element_id, $module_name, $lang);
	   }
	   return true;
	}
	
    /**
    * Создание поискового индекса для заданных элементов модуля.
    * 
    * @param object $th объект системы
    * @param string $table  название модуля
    * @param array $element_ids массив элементов
    * 
    * @return boolean true
    */
	protected static function create_search_index($th, $table, $element_ids)
	{
	   Customization::inc('modules/'.self::$params['module'].'/'.self::$params['module'].'.search.php');

	   $config = self::get_config(self::$params['module']);
	   $module = $table;
	   $lang = _LANG ? _LANG : 1;
	   
	   foreach ($element_ids as $element_id)
	   {
		  $url = self::get_url($th, $module, $element_id);
		  
		  $element_for_indexation = new Element_for_indexation_act($element_id, $module, $url,
		                                                       $config,  $lang); 
		  try
		  {
			 $search_index = new Search_index($element_for_indexation, $th->diafan);
		  }
		  catch(Exception $e)
		  {
			 echo $e;
			 exit;
		  }
		  
		  try
		  {
			 $search_index->add();
		  }
		  catch(Exception $e)
		  {
			 echo $e;
			 exit;
		  }
	   }
	}
    
    /**
    * Получение конфигурации модуля, к которому относится элемент.
    * 
    * @param string $module название модуля
    * 
    * @return array $config
    */
    protected static function get_config($module)
    {
        if(! empty(self::$config[$module]))
        {
            return self::$config[$module];
        }

        $config_classname = $module.'_search_config';
        $config = new $config_classname();
        self::$config[$module] = $config->config;

        return  self::$config[$module];
    }

    
    /**
    * Получение url элемента
    * 
    * @return string $url
    */
    protected static function get_url($th, $module, $element_id)
    {
	   $url = '';

	   if ($_GET["rewrite"] || defined('_LANG') && _LANG)
	   {
		  $rew = explode('/', $_GET["rewrite"], 2);
		  foreach ($th->diafan->language_all as $l => $langarr)
		  {
			 if (! defined('_LANG') && in_array($rew[0], $langarr) || defined('_LANG') && $langarr[0] == _LANG)
			 {
				$url .= ! empty($langarr[1])? $langarr[1].'/' : '';
				break;
			 }
		  }
	   }

	   $site_id = ! empty(self::$params['site']) ? self::$params['site'] : $th->diafan->cid;

	   if(self::$params['config_module']['category'])
	   {
		  $cat_id =  $element_id;
		  $element_id = 0;
	   }
	   else
	   {
		 $lang = _LANG ? _LANG : 0;
           if($th->diafan->configmodules ('cat', self::$params['module'], $site_id, $lang))
           {
               $cat_id = DB::query_result("SELECT cat_id FROM {%s} WHERE id=%d", $module, $element_id);
           }
           else
           {
               $cat_id = 0;
           }
	   }
        if(! ($element_id == 1 && self::$params['module'] == 'site'))
        {
            $url .= $th->diafan->_route->link($site_id, self::$params['module'], $cat_id, $element_id);
        }

	   return $url;
    }
}