<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.0
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Search_index
 *
 * Создание удаление поискового индекса
 */
class Search_index
{
    protected $config;

    protected $search_words_processor;

    protected $element_for_indexation;
    
    public function __construct(Element_for_indexation $element_for_indexation, $diafan)
    {
        $this->element_for_indexation = $element_for_indexation;
        $this->diafan = $diafan;
    }
    
    /**
    * Добавление одного элемента в поисковый индекс.
    * 
    * 
    * @return boolean true
    */
    public function add()
    {
	  self::delete($this->element_for_indexation->id, $this->element_for_indexation->module,
		          $this->element_for_indexation->lang);
	   	   
	   $values_for_indexation = $this->element_for_indexation->get_values_for_indexation();
	   
	   $index_words = $this->diafan->_search->prepare($values_for_indexation);
	   if(empty($index_words))
	   {
		  return false;
	   }
	   
	   if(! DB::query("INSERT INTO {search_results} (name, url, snippet, module, element_id, rating, lang_id)
		         VALUES('%s', '%s', '%s', '%s', %d, %d, %d)",
                 $this->diafan->short_text($this->element_for_indexation->get_name()),
                 $this->element_for_indexation->get_url(),
                 $this->diafan->short_text($this->element_for_indexation->get_snippet()),
                 $this->element_for_indexation->module,
                 $this->element_for_indexation->id,
                 $this->element_for_indexation->get_rating(),
                 $this->element_for_indexation->lang))
	   {
		  throw new Exception('Ошибка записи в БД');
	   }
	   $result_id = DB::last_id('search_results');
	   
	   $keywords_already_exists = array();
	   
	   $keywords_result = DB::query("SELECT id, keyword FROM {search_keywords} WHERE keyword IN ('"
		                           .implode("', '", $index_words)."')");
	   
	   while ($row = DB::fetch_array($keywords_result))
	   {
		  $keywords_already_exists[$row['id']] = $row['keyword'];
	   }
	   $new_words = array();
	   foreach($index_words as $word)
	   {
		  if(! in_array($word, $keywords_already_exists))
		  {
			 $new_words[] = $word;
		  }
	   }
	   if($new_words)
	   {
			$query = "";
			$k = 0;
			foreach ($new_words as $word)
			{
					if(! $query)
					{
						$query = "INSERT INTO {search_keywords} (keyword) VALUES ('".$word."')";
					}
					else
					{
						$query .= ", ('".$word."')";
					}
					$k++;
					if($k == 100)
					{
						DB::query($query);
						$query = '';
						$k = 0;
					}
			}
			if($query)
			{
				DB::query($query);
			}
			$keywords_result = DB::query("SELECT id, keyword FROM {search_keywords} WHERE keyword IN ('"
										.implode("', '", $new_words)."')");

			while ($row = DB::fetch_array($keywords_result))
			{
				$keywords_already_exists[$row['id']] = $row['keyword'];
			}
	   }
	   $query = "";
	   $k = 0;
	   foreach ($keywords_already_exists as $keyword_id => $dummy)
	   {
			if(! $query)
			{
				$query = "INSERT INTO {search_index} (keyword_id, result_id) VALUES (".$keyword_id.", ".$result_id.")";
			}
			else
			{
				$query .= ", (".$keyword_id.", ".$result_id.")";
			}
			$k++;
			if($k == 100)
			{
				DB::query($query);
				$query = '';
				$k = 0;
			}
	   }
	   if($query)
	   {
		   DB::query($query);
	   }
	   return true;
    }
    
    /**
    * Удаление одного элемента из поискового индекса.
    * 
    * @param integer $id идентификатор элемента
    * @param string $module название модуля
    * @param integer $lang идентификатор языка
    * 
    * @return boolean true
    */
    public static function delete($id, $module, $lang)
    {
	   $result_id = DB::query_result("SELECT id FROM {search_results} WHERE element_id=%d AND module='%s'
		                             AND lang_id=%d",
	                                 $id, $module, $lang);

	   if(empty($result_id))
	   {
		  return false;
	   }
		$result = DB::query("SELECT DISTINCT i.keyword_id FROM {search_index} as i"
				." INNER JOIN {search_index} AS i2 ON i.keyword_id=i2.keyword_id"
				." WHERE i.result_id=%d GROUP BY i.keyword_id HAVING count(i2.keyword_id)<2", $result_id);
	   while($row = DB::fetch_array($result))
	   {
		   $keys[] = $row["keyword_id"];
	   }

	   DB::query("DELETE FROM {search_results} WHERE id=%d", $result_id);
	   DB::query("DELETE FROM {search_index} WHERE result_id=%d", $result_id);
	   if($keys)
	   {
			$query = "";
			$k = 0;
			foreach ($keys as $key)
			{
					if(! $query)
					{
						$query = "DELETE FROM {search_keywords} WHERE id IN (".$key;
					}
					else
					{
						$query .= ", ".$key;
					}
					$k++;
					if($k == 100)
					{
						DB::query($query.")");
						$query = '';
						$k = 0;
					}
			}
			if($query)
			{
				DB::query($query.")");
			}
	   }

	   return true;
    }
}


/**
 * Element_for_indexation
 *
 * Элемент, добавляемый в индекс при сохранении.
 */
class  Element_for_indexation
{
    public $config;

    public $module;

    public $id;

    public $lang;

    public $url;
    
    /**
    * Создание элемента
    * 
    * @param integer $id идентификатор элемента
    * @param string $module название модуля
    * @param string $url url-адрес элемента
    * @param array $config конфигурация модуля, к которому относится элемент, для работы с поисковым индексом
    * @param integer $lang идентификатор языка
    * 
    * @return boolean true
    */
    public function __construct($id, $module, $url, $config, $lang = 1)
    {
        $this->id = $id;
        $this->module = $module;
        $this->lang = $lang;
        $this->config = $config;
        $this->url = $url;
    }
    
    /**
    * Получении имени/названия элемента
    * 
    * @return string $name
    */
    public function get_name()
    {
       return empty($_POST[$this->config['name_field']]) ? false : $_POST[$this->config['name_field']];
    }
    
    /**
    * Получении  url-адрес элементаэлемента
    * 
    * @return string $url
    */
    public function get_url()
    {
        return $this->url;
    }
    
    /**
    * Получении  сниппета
    * 
    * @return string $snippet
    */
    public function get_snippet()
    {
        $snippet = '';

        foreach($this->config['snippet_fields'] as $field)
        {
            if(! empty($_POST[$field]))
            {
                $snippet .= ($snippet ? ' ' : '').$_POST[$field];
            }
        }

        return $snippet;
    }
    
    /**
    * Получении  рейтинга
    * 
    * @return integer $rating
    */
    public function get_rating()
    {
       return  empty($this->config[ 'rating'])? 1 : $this->config[ 'rating'];
    }
    
   /**
    * Получении  значений, учитываемых при добавлении в индекс.
    * 
    * @return string $values_for_indexation
    */
    public function get_values_for_indexation() 
    {
	   $values_for_indexation = '';
	   
	   foreach($this->config['fields'] as $field_name)
	   {
		  if(! empty($_REQUEST[$field_name]))
		  {
			 $values_for_indexation .= ' '.$_REQUEST[$field_name];
		  }
	   }
	   return $values_for_indexation;
    }
}

/**
 * Element_for_indexation_act
 *
 * Элемент, добавляемый/удаляемый  из индекса при активации/деактивации/удалении элемента.
 */
class Element_for_indexation_act extends Element_for_indexation
{
    /**
    * Получении имени/названия элемента
    * 
    * @return string $name
    */
    public function get_name()
    {
        $name_field = $this->config['name_field'].($this->lang != 1 ? $this->lang :'');

        $row = DB::fetch_array(DB::query("SELECT %s FROM {%s} WHERE id=%d",
                               $name_field, $this->module, $this->id));
        return $row[$name_field];
    }
    
    /**
    * Получении  сниппета
    * 
    * @return string $snippet
    */
    public function get_snippet()
    {
        $snippet_fields = array();
	   $lang = ($this->lang != 1 ? $this->lang :'');
        
	   foreach($this->config['snippet_fields'] as $field)
        {
            $snippet_fields[] = $field.$lang;
        }

        $row = DB::fetch_array(DB::query("SELECT ".(implode(', ', $snippet_fields))
                               ." FROM {%s} WHERE id=%d", $this->module, $this->id));

        return implode(' ', $row);
    }
    
    /**
    * Получении  значений, учитываемых при добавлении в индекс.
    * 
    * @return string $values_for_indexation
    */
    public function get_values_for_indexation() 
    {
	   $indexation_fields = array();
	   $lang = ($this->lang != 1 ? $this->lang :'');
        
	   foreach($this->config['fields'] as $field)
        {
            $indexation_fields[] = $field.$lang;
        }

        $row = DB::fetch_array(DB::query("SELECT ".(implode(', ', $indexation_fields))
                               ." FROM {%s} WHERE id=%d", $this->module, $this->id));

        return implode(' ', $row);
    }
    
}


