<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.0
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Shop
 *
 * Контроллер модуля "Магазин"
 */
class Shop extends Controller
{
	/**
	 * Инициализация модуля
	 * 
	 * @return void
	 */
	public function init()
	{
		$this->rewrite_variable_names = array('page', 'show', 'cat', 'sort', 'param');
		$this->diafan->rewrite_variable_names = $this->rewrite_variable_names;

		$model = new Shop_model($this->diafan);

		if ($this->diafan->show)
		{
			$model->id();
		}
		elseif ($this->diafan->param)
		{
			$model->list_param();
		}
        elseif(isset($_GET["action"]) && $_GET["action"] === 'search')
        {
            $model->list_search();
        }
		elseif(isset($_GET["action"]) && $_GET["action"] === 'compare')
        {
            $model->compare();
        }
        elseif(isset($_GET["action"]) && $_GET["action"] === 'file' && isset($_GET["code"]))
        {
            $model->file_get();
        }
		elseif (! $this->diafan->configmodules("cat"))
		{
			$model->list_();
		}
		elseif (! $this->diafan->cat)
		{
			$model->first_page();
		}
		else
		{
			$model->list_category();
		}
		$this->result = $model->get_result();
		$this->get_global_variables();
	}

	/**
	 * Выводит контент модуля
	 * 
	 * @return boolean true
	 */
	public function show_module()
	{
		if (! empty($this->result["err"]))
		{
			echo $this->result["err"];
			return false;
		}
		$this->diafan->_tpl->get($this->view, 'shop', $this->result);

		return true;
	}

	/**
	 * Шаблонная функция: блок товаров
	 * 
	 * @param array $attributes атрибуты шаблонного тэга
	 * @return boolean
	 */
	public function show_block($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'only_shop', 'count', 'cat_id', 'site_id', 'images', 'sort', 'param', 'template',
                'hits_only', 'action_only', 'new_only');

		$count   = $attributes["count"] ? intval($attributes["count"]) : 3;
		$cat_id  = intval($attributes["cat_id"]);
		$site_id = intval($attributes["site_id"]);
		$images  = intval($attributes["images"]);
		$param   = $attributes["param"];
		$sort    = in_array($attributes["sort"], array("date", "rand", "price", "sale")) ? $attributes["sort"] : "date";
		$only_shop = $attributes["only_shop"];
        $hits_only = (bool) $attributes["hits_only"];
		$action_only  = (bool) $attributes["action_only"];
		$new_only   = (bool) $attributes["new_only"];

		if ($only_shop && ($this->diafan->module != "shop" || $site_id && $this->diafan->cid != $site_id))
			return false;

		Customization::inc('modules/shop/shop.model.php');
		$model = new Shop_model($this->diafan);
		$model->show_block($count, $cat_id, $site_id, $sort, $images, $param, $hits_only,
                $action_only, $new_only);
		$result = $model->get_result();

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_'.$attributes["template"], 'shop', $result))
		{
			$this->diafan->_tpl->get('show_block', 'shop', $result);
		}

		return true;
	}

	/**
	 * Шаблонная функция: блок связанных товаров
	 * 
	 * @param array $attributes атрибуты шаблонного тэга
	 * @return boolean
	 */
	public function show_block_rel($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'count', 'images', 'template');

		$count   = $attributes["count"] ? intval($attributes["count"]) : 3;
		$images  = intval($attributes["images"]);

		if ($this->diafan->module != "shop" || ! $this->diafan->show)
			return false;

		Customization::inc('modules/shop/shop.model.php');
		$model = new Shop_model($this->diafan);
		$model->show_block_rel($count, $images);
		$result = $model->get_result();

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_rel_'.$attributes["template"], 'shop', $result))
		{
			$this->diafan->_tpl->get('show_block_rel', 'shop', $result);
		}

		return true;
	}

	/**
	 * Шаблонная функция: блок товаров, которые обычно покупают с текущим товаром
	 * 
	 * @param array $attributes атрибуты шаблонного тэга
	 * @return boolean
	 */
	public function show_block_order_rel($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'count', 'images', 'template');

		$count   = $attributes["count"] ? intval($attributes["count"]) : 3;
		$images  = intval($attributes["images"]);

		if ($this->diafan->module != "shop" || ! $this->diafan->show)
			return false;

		Customization::inc('modules/shop/shop.model.php');
		$model = new Shop_model($this->diafan);
		$model->show_block_order_rel($count, $images);
		$result = $model->get_result();

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_block_order_rel_'.$attributes["template"], 'shop', $result))
		{
			$this->diafan->_tpl->get('show_block_order_rel', 'shop', $result);
		}

		return true;
	}

	/**
	 * Шаблонная функция: форма поиска по товарам
	 * 
	 * @param array $attributes атрибуты шаблонного тэга
	 * @return boolean
	 */
	public function show_search($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'only_shop', 'cat_id', 'site_id', 'template');

		$only_shop = $attributes["only_shop"];
		$cat_id    = $attributes["cat_id"] === 'current' ? 'current' : intval($attributes["cat_id"]);
		$site_id   = intval($attributes["site_id"]);

		if ($only_shop && ($this->diafan->module != "shop" || $site_id && $this->diafan->cid != $site_id))
			return false;

		Customization::inc('modules/shop/shop.model.php');
		$model = new Shop_model($this->diafan);
		$result = $model->show_search($cat_id, $site_id);

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_search_'.$attributes["template"], 'shop', $result))
		{
			$this->diafan->_tpl->get('show_search', 'shop', $result);
		}

		return true;
	}

	/**
	 * Шаблонная функция: форма активации купона
	 * 
	 * @param array $attributes атрибуты шаблонного тэга
	 * @return boolean
	 */
	public function show_add_coupon($attributes)
	{
		$attributes = $this->get_attributes($attributes, 'template');

		if (! $this->diafan->_user->id)
			return false;

		Customization::inc('modules/shop/shop.model.php');
		$model = new Shop_model($this->diafan);
		$result = $model->show_add_coupon();

		// нельзя добавить два купона
		if ($result["count"])
			return false;

		if (! $attributes["template"] || ! $this->diafan->_tpl->get('show_add_coupon_'.$attributes["template"], 'shop', $result))
		{
			$this->diafan->_tpl->get('show_add_coupon', 'shop', $result);
		}

		return true;
	}
}