<?php
/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.0
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */

if (! defined('DIAFAN'))
{
	include dirname(dirname(dirname(__FILE__))).'/includes/404.php';
}

/**
 * Trash_admin
 *
 * Корзина с удаленными элементами
 */
class Trash_admin extends Frame_admin
{
	/**
	 * @var string название файла. используется в имени констант, определяющих названия полей
	 */
	public $file	=	'trash';

	/**
	 * @var string таблица в базе данных
	 */
	public $table	=	'trash'; 

	/**
	 * @var string таблица в базе данных
	 */
	protected $name = "id, ''";

	/**
	 * Конструктор класса
	 * 
	 * @return void
	 */
	public function __construct($diafan)
	{
		$this->diafan = $diafan;

		$this->config_module = array(
			'del'		=> 1,	//удалить
			'datetime'	=> 1,	//показывать дату и время в списке
			'parent'	=> 1	//содержит вложенности
		);

		//выводить в списке содержание полей:
		$this->config_other_row = array(
			'element_id',
			'module_name',
			'table_name',
			'user_id'
		);
		
		$this->group_action=array("restore"=>"Восстановить");
	}

	/**
	 * Выводит список удаленных в коризун элементов
	 * @return boolean true
	 */
	public function show_module()
	{
		$this->list_row();
		
		if (! $this->count)
		{
			echo _ADMIN_TRASH_NONE;
		}

		return true;
	}

	/**
	 * Восстанавливает элементы из корзины
	 * @return boolean
	 */
	public function restore_module()
	{
		// Прошел ли пользователь проверку идентификационного хэша
		if (! $this->diafan->_user->checked)
		{
			$this->diafan->redirect(URL);
			return false;
		}

		if (! empty($_POST["id"]))
		{
			$ids = array($_POST["id"]);
		}
		else
		{
			$ids = $_POST["ids"];
		}
		foreach ($ids as $id)
		{
			$id = intval($id);
			$this->restore_module_id($id);
		}

		include_once ABSOLUTE_PATH.'plugins/json.php';
		$result["redirect"] = URL;
		echo to_json($result);
	}

	/**
	 * Восстанавливает элемент из корзины
	 * 
	 * @param integer $id номер восстанавливаемого элемента
	 * @return boolean
	 */
	public function restore_module_id($id)
	{
		$row = DB::fetch_array(DB::query("SELECT table_name, element_id, module_name, id FROM {trash} WHERE id = %d", $id));

		if (! $row)
		{
			return false;
		}

		//проверка прав пользователя на удаление
		if ($this->diafan->admin_roles('del', $row["module_name"]))
		{
			return false;
		}
		$this->diafan->_cache->delete('', $row["module_name"]);

		$this->restore_elements($row["table_name"], $row["element_id"], $row["id"]);

		return true;
	}

	/**
	 * Восстанавливает связанные элементы из корзины
	 *
	 * @param string $table название таблицы
	 * @param integer $id номер восстанавливаемого элемента
	 * @param integer $trash_id номер записи в корзине
	 * @return boolean true
	 */
	private function restore_elements($table, $id, $trash_id)
	{
		DB::query("UPDATE {".$table."} SET trash='0' WHERE id=%d", $id);
		DB::query("DELETE FROM {trash} WHERE id=%d", $trash_id);
		DB::query("DELETE FROM {trash_parents} WHERE parent_id=%d AND element_id=%d", $trash_id, $trash_id);

		$result = DB::query("SELECT element_id, table_name,id FROM {trash} WHERE parent_id=%d", $trash_id);

		while ($row = DB::fetch_array($result))
		{
			$this->restore_elements($row["table_name"], $row["element_id"], $row["id"]);
		}
		if (! strstr($table, '_parents'))
		{
			$del_row = DB::fetch_array(DB::query("SELECT * FROM {".$table."} WHERE id=%d", $id));
			if (! empty($del_row["parent_id"]))
			{
				$count = DB::query_result("SELECT COUNT(*) FROM {".$table."} WHERE trash='0' AND parent_id=%d", $del_row["parent_id"]);
				DB::query("UPDATE {".$table."} SET count_children=%d WHERE id=%d", $count, $del_row["parent_id"]);
			}
		}

		return true;
	}

	/**
	 * Удаляет элемент или элементы из корзины
	 * @return boolean
	 */
	public function del_module()
	{
		// Прошел ли пользователь проверку идентификационного хэша
		if (! $this->diafan->_user->checked)
		{
			$this->diafan->redirect(URL);
			return false;
		}

		if (! empty($_POST["id"]))
		{
			$ids = array($_POST["id"]);
		}
		else
		{
			$ids = $_POST["ids"];
		}
		foreach ($ids as $id)
		{
			$id = intval($id);
			$this->del_module_id($id);
		}

		include_once ABSOLUTE_PATH.'plugins/json.php';
		$result["redirect"] = URL;
		echo to_json($result);

		return true;
	}

	/**
	 * Удаляет элемент из корзины
	 * 
	 * @param integer $id номер удаляемого элемента
	 * @return boolean
	 */
	public function del_module_id($id)
	{
		
		$row = DB::fetch_array(DB::query("SELECT table_name, element_id, id FROM {trash} WHERE id = %d", $id));

		if (! $row)
		{
			return false;
		}
		$this->del_from_trash($row["table_name"], $row["element_id"], $row["id"]);
		return true;
	}

	/**
	 * Выводит название удаленного объекта в списке
	 * @return string
	 */
	protected function other_row_element_id($row)
	{
		$data = '<td>';
		switch($row["table_name"])
		{
			case "images":
				$image_row = DB::fetch_array(DB::query("SELECT name, module_name FROM {".$row["table_name"]."} WHERE id='%d'", $row["element_id"]));

				if ($image_row && file_exists(ABSOLUTE_PATH.USERFILES."/".$image_row["module_name"]."/small/".$image_row["name"]))
				{
					$data .= '<img src="http://'.BASE_URL.'/'.USERFILES.'/'.$image_row["module_name"]."/small/".$image_row["name"].'" border="0">';
				}
				break;

			case "users":
				$user = DB::fetch_array(DB::query("SELECT fio, name FROM {".$row["table_name"]."} WHERE id='%d'", $row["element_id"]));
				$data .= $user["fio"].' ('.$user["name"].')';
				break;

			case "tags":
				$text = DB::query_result("SELECT [name] FROM {tags_name} WHERE id IN ("
							." SELECT tags_name_id FROM {".$row["table_name"]."} WHERE id='%d')", $row["element_id"]);
				$data .= ($text ? $this->diafan->short_text($text) : $row["element_id"]);
				break;

			case "rating":
				$rating = DB::query_result("SELECT rating FROM {".$row["table_name"]."} WHERE id='%d'", $row["element_id"]);
				$data .= ($rating ? $rating : $row["element_id"]);
				break;

			case "feedback":
				$data .= _ADMIN_FEEDBACK_MESSAGE;
				break;

			case "feedback_param_element":
				$param = DB::fetch_array(DB::query("SELECT param_id, value FROM {".$row["table_name"]."} WHERE id='%d'", $row["element_id"]));
				$param_name = DB::query_result("SELECT [name] FROM {feedback_param} WHERE id='%d'", $param["param_id"]);
				$data .= $param_name.': '.$param["value"];
				break;

			case "shop_order":
				$data .= _ADMIN_SHOPORDER_NUMBER.' '.$row["element_id"];
				break;

			case "shop_rel":
				$data .= _ADMIN_GOODS_REL;
				break;

			case "shop_param_element":
				$param = DB::fetch_array(DB::query("SELECT param_id, value FROM {".$row["table_name"]."} WHERE id='%d'", $row["element_id"]));
				$param_name = DB::query_result("SELECT [name] FROM {shop_param} WHERE id='%d'", $param["param_id"]);
				$data .= $param_name.': '.$param["value"];
				break;

			case "rewrite":
				$data .= _ADMIN_REWRITE;
				break;

			default:
				$element = DB::fetch_array(DB::query("SELECT * FROM {".$row["table_name"]."} WHERE id='%d'", $row["element_id"]));
				$name = $row["element_id"];
				if (! empty($element["name"]))
				{
					$name = $element["name"];
				}
				elseif (! empty($element["text"]))
				{
					$name = $this->diafan->short_text($element["text"], 50);
				}
				if ($row["table_name"] == "shop_price")
				{
					return $element["price"].' '.$this->diafan->configmodules("currency", "shop");
				}
				elseif (strstr($row["table_name"], '_parents'))
				{
					$name = _ADMIN_PARENTS_REL;
				}
				elseif (strstr($row["table_name"], '_category_rel'))
				{
					$name = _ADMIN_CATEGORY_REL;
				}
				$data .= $name;
		}
		return $data.'</td>';
	}

	/**
	 * Выводит название модуля удаленного объекта в списке
	 * @return string
	 */
	protected function other_row_table_name($row)
	{
		if (strstr($row["table_name"], '_'))
		{
			$name_array = explode('_', $row["table_name"]);
			foreach ($name_array as $n)
			{
				$const = strtoupper('_LANG_'.$n);
				if (defined($const))
				{
					$name[] = constant($const);
				}
			}
			if (! empty($name))
			{
				$name = implode(', ', $name);
			}
			else
			{
				$name = $row["table_name"];
			}
		}
		else
		{
			$const = strtoupper('_LANG_'.$row["table_name"]);
			$name = defined($const) ? constant($const) : $row["table_name"];
		}
		return '</td><td>'.$name
		.'</td><td class="restore-action">'.(! $row["parent_id"] ? '<a href="javascript:void(0);" action="restore">'._LANG_RESTORE.'</a>':'');
	}

	/**
	 * Выводит название модуля удаленного объекта в списке
	 * @return string
	 */
	protected function other_row_module_name($row)
	{
		return '';
	}

	/**
	 * Выводит имя пользователя, удалившего элемент
	 * @return string
	 */
	protected function other_row_user_id($row)
	{
		return '</td><td>'
		.(! $row["parent_id"] && $row["user_id"] ? '<a href="'.BASE_PATH_HREF.'users/edit'.$row["user_id"].'/">'.DB::title("users", $row["user_id"], "fio").'</a>':'');
	}

	/**
	 * Редактирование поля "Пользователь"
	 * @return boolean true
	 */
	protected function edit_user_id_module()
	{
		if ($this->value)
		{
			echo '
			<tr>
				<td align="right">'.$this->variable_names().'</td>
				<td>
					<a href="'.BASE_PATH_HREF.'users/edit'.$this->value.'/">'.DB::title("users", $this->value, "fio").'</a>'
					.$this->show_help_row().'
				</td>
			</tr>';
		}

		return true;
	}
}